"""
Given an HTML string, returns a list of local images
and source adjusted to refer that by cid:


Assumptions - all local images are in the same folder.
May be we should just go ahead and download all images before
packing the mail.
"""

__version__ = "$Revision: 1.1.1.1 $"
# $Source: /home/vsbabu/repository/python/compmail/MyHTMLProcessor.py,v $

from sgmllib import SGMLParser
import os.path
import string

base_href="http://localhost"

class MyHTMLProcessor(SGMLParser):
    def reset(self):
        self.pieces = []
        self.images = []
        SGMLParser.reset(self)

    def unknown_starttag(self, tag, attrs):
        strattrs = "".join([' %s="%s"' % (key, value) for key, value in attrs])
        self.pieces.append("<%(tag)s%(strattrs)s>" % locals())

    def unknown_endtag(self, tag):
        self.pieces.append("</%(tag)s>" % locals())

    def handle_charref(self, ref):
        self.pieces.append("&#%(ref)s;" % locals())

    def handle_entityref(self, ref):
        self.pieces.append("&%(ref)s" % locals())
        if htmlentitydefs.entitydefs.has_key(ref):
            self.pieces.append(";")

    def handle_data(self, text):
        self.pieces.append(text)

    def handle_comment(self, text):
        self.pieces.append("<!--%(text)s-->" % locals())

    def handle_pi(self, text): 
        self.pieces.append("<?%(text)s>" % locals())

    def handle_decl(self, text):
        self.pieces.append("<!%(text)s>" % locals())

    def start_img(self, attrs):
        tag = "img"
        imgsrc = [v for k,v in attrs if k=='src'][0]
        newtag = None
        if imgsrc:
            #check if it is an image in another site
            #we don't want that either. this not foolproof
            #at all. I'm assuming that all local images
            #are in the same folder - which is true in my case
            if imgsrc[:4] == "http":
                if imgsrc.find(base_href) == 0:
                    self.images.append(os.path.basename(imgsrc))
                    newtag = "cid:" + os.path.basename(imgsrc)
            else:
                    self.images.append(os.path.basename(imgsrc))
                    newtag = "cid:" + os.path.basename(imgsrc)
        strattrs = ""
        for (key,value) in attrs:
            if key == 'src' and newtag is not None:
                value = newtag
            strattrs = strattrs + " " + key  + '="' + value + '"'
        self.pieces.append("<%(tag)s%(strattrs)s>" % locals())
    
    def output(self):
        """Return processed HTML as a string"""
        return "".join(self.pieces)

    def local_images(self):
        """Return a list of local images"""
        return unique(self.images)

def unique(s):
    """Return a list of the elements in s, but without duplicates.

    For example, unique([1,2,3,1,2,3]) is some permutation of [1,2,3],
    unique("abcabc") some permutation of ["a", "b", "c"], and
    unique(([1, 2], [2, 3], [1, 2])) some permutation of
    [[2, 3], [1, 2]].

    For best speed, all sequence elements should be hashable.  Then
    unique() will usually work in linear time.

    If not possible, the sequence elements should enjoy a total
    ordering, and if list(s).sort() doesn't raise TypeError it's
    assumed that they do enjoy a total ordering.  Then unique() will
    usually work in O(N*log2(N)) time.

    If that's not possible either, the sequence elements must support
    equality-testing.  Then unique() will usually work in quadratic
    time.
    """

    n = len(s)
    if n == 0:
        return []

    # Try using a dict first, as that's the fastest and will usually
    # work.  If it doesn't work, it will usually fail quickly, so it
    # usually doesn't cost much to *try* it.  It requires that all the
    # sequence elements be hashable, and support equality comparison.
    u = {}
    try:
        for x in s:
            u[x] = 1
    except TypeError:
        del u  # move on to the next method
    else:
        return u.keys()

    # We can't hash all the elements.  Second fastest is to sort,
    # which brings the equal elements together; then duplicates are
    # easy to weed out in a single pass.
    # NOTE:  Python's list.sort() was designed to be efficient in the
    # presence of many duplicate elements.  This isn't true of all
    # sort functions in all languages or libraries, so this approach
    # is more effective in Python than it may be elsewhere.
    try:
        t = list(s)
        t.sort()
    except TypeError:
        del t  # move on to the next method
    else:
        assert n > 0
        last = t[0]
        lasti = i = 1
        while i < n:
            if t[i] != last:
                t[lasti] = last = t[i]
                lasti += 1
            i += 1
        return t[:lasti]

    # Brute force is all that's left.
    u = []
    for x in s:
        if x not in u:
            u.append(x)
    return u

if __name__=='__main__':
    html = open("test.html", "r").read()
    parser = MyHTMLProcessor()
    parser.feed(html)
    parser.close()
    print parser.local_images()
    print parser.output()

